import router from '@/router';
import { IdPool } from '@/utils';
import type { PageErrorReason } from '@/views/ErrorPage.vue';
import { defineStore } from 'pinia';
import { computed } from 'vue';
import {
  type RouteLocationNormalized,
  type RouteLocationRaw,
  type RouteRecordRaw,
} from 'vue-router';

const tempErrorRouteName = '_TempError';
export const usePageStore = defineStore('page-store', () => {
  const routeIdPool = new IdPool();
  const routeIds = routeIdPool.usedIdSet;
  const pageLoadingCount = computed(() => routeIds.size);

  function setNewRouteId(route: RouteLocationNormalized) {
    return (route.meta.routeId = routeIdPool.newId());
  }

  function removeRouteId(route: RouteLocationNormalized) {
    const { routeId } = route.meta;
    if (routeId === undefined) {
      return;
    }
    routeIdPool.removeId(routeId);
  }

  function createTempErrorRoute(
    reason: Exclude<
      PageErrorReason,
      {
        type: 'notFound';
      }
    >,
    to: RouteLocationNormalized,
  ): RouteLocationRaw {
    const tempErrorRoute: RouteRecordRaw = {
      path: to.fullPath,
      name: tempErrorRouteName,
      component: () => import('@/views/ErrorPage.vue'),
      props: reason,
    };
    if (router.hasRoute(tempErrorRouteName)) {
      router.removeRoute(tempErrorRouteName);
    }
    router.addRoute(tempErrorRoute);
    return { name: tempErrorRouteName, replace: true };
  }

  return {
    pageLoadingCount,
    setNewRouteId,
    removeRouteId,
    createTempErrorRoute,
  };
});
